/*-----------------------------------------------------------------------*/
/*  (C) Copyright Institute of Astronomy ETHZ 8092 Zuerich Switzerland   */
/*-----------------------------------------------------------------------*/
/*  Programmname: digitizer.cpp                                          */
/*                                                                       */
/*  Revision: V1.0     Date: 26.12.2012    Autor: Chr. Monstein          */
/*                                                                       */
/*  Purpose: Noise Figure Measurement of a CALLISTO                      */
/*                                                                       */
/*  Compiler: BORLAND C++Builder 6                                       */
/*-----------------------------------------------------------------------*/

// Created by: Chr. Monstein 26.12.2012
// Updated by: Chr. Monstein 30.12.2012 new data save
//---------------------------------------------------------------------------

#include <vcl.h>
#pragma hdrstop
#include <stdio.h>

#include "DigitizerUnit.h"
#include "RXRS232.H"

#include "mainunit.h"
//---------------------------------------------------------------------------
#pragma package(smart_init)
#pragma resource "*.dfm"
TForm3 *Form3;
AnsiString TimerSt;   // to be logged...
AnsiString DatumSt;   // to be logged...
float Tact;
float t; //Decimal Time
Word Year, Month, Day;
Word hour, minute, second, msec;

char logpath[240];
char filename[240];
char ending[10];
char delimiter[5];
bool logfirst=true;
int v1; // voltage / mV
float vr; // voltage ingtegrated normalized

int NN; // number of integrations
long n=0; // pixel count

#define YTbuflen 1000
float *Yt_ringbuff;
long Yt_in=0;  //
bool disp;

extern float fnom;   // import from mainunit.cpp
extern int pwm;      // import from mainunit.cpp
extern int focus;    // import from mainunit.cpp
extern int sampling; // import from mainunit.cpp

// import form RXRS232.cpp
extern TWrite *RX_SerialThread; // RX activities
extern Byte RX_ringbuff[RXbuflen];
extern long RX_in, RX_out;

//---------------------------------------------------------------------------
__fastcall TForm3::TForm3(TComponent* Owner)
        : TForm(Owner)
{
}
//---------------------------------------------------------------------------
void __fastcall TForm3::Init(void)
{
    char st[80];

    Yt_ringbuff = new float[YTbuflen];
    Button1Click(0);
    
    disp = true;

    RX_SerialThread->Send("%2\r"); // set fixed format mV

    sprintf(st,"%u",pwm);
    Edit2->Text = st;
    sprintf(st,"O%03u\r",pwm); // gain control
    RX_SerialThread->Send(st);

    sprintf(st,"%02u",focus);
    Edit3->Text = st;
    sprintf(st,"fs%02i\r",focus);
    RX_SerialThread->Send(st);  // switch cold

    sprintf(st,"%i",sampling);
    Edit4->Text = st;
    NN = sampling/(2*Timer1->Interval); // t/(2*dt)

    ProgressBar1->Min      =    0;
    ProgressBar1->Max      = 2500;
    ProgressBar1->Position =    0;

    sprintf(st,"%6.3f",fnom);
    Edit1->Text = st;

    //Timer1->Enabled = true;  // Start

    MyClock();
    sprintf(filename,"%sDigitizer_%04u%02u%02u_%02u%02u%02u.%s",
            logpath,Year,Month,Day,hour, minute, second,ending);

}
//---------------------------------------------------------------------------
void __fastcall TForm3::MyClock(void)
{
    TDateTime dtPresent = Now();
    DecodeDate(dtPresent, Year, Month, Day);
    DatumSt = DateToStr(dtPresent);
    DecodeTime(dtPresent, hour, minute, second, msec);
    TimerSt = TimeToStr(Time()); // needed for logfiles
    Tact = hour + ((float)minute)/60 + ((float)second)/3600 + ((float)msec)/3600/1000;
}
//---------------------------------------------------------------------------
void __fastcall TForm3::Timer1Timer(TObject *Sender)
{
    short status;
    char st[80];
    char data[80];
    static int state = 0;
    static int ii=0;
    static float vi = 0.0;

    char ch;

    switch(state) // execute every x msec
    {
        case 0: // ask for ADC
        {
            ieee->Reset(); // delete rs232 buffer
            RX_SerialThread->Send("A0\r"); // detector voltage
            MyClock(); // get time and date
            state++;
            break;
        }

        case 1: // wait for incoming data
        {
            int i = 0;
            do
            {
                ch = RX_ringbuff[RX_out];
                RX_out = (RX_out+1)%RXbuflen;
                if (ch==13) break;
                data[i++]=ch;
                if (i>20)
                {
                   state = 0;
                   break; // emergeny exit
                }
            } while (1);
            data[i]='\0';

            int p=0;
            if (strstr(data,"$CRX:ADC0="))
            {
              p = sscanf(data,"$CRX:ADC0=%u",&v1);
              if (p==1)
              {
                  vi = vi + float(v1); // integration
                  ii++;
                  if (ii >= NN)
                  {
                     vr = vi/float(NN);
                     ii = 0;
                     vi = 0.0;
                     n++;
                     if ((n%500)==0) Memo1->Clear();
                     sprintf(st,"Voltage:%6.3f mV",vr);
                     Label1->Caption = st;  // voltage
                     ProgressBar1->Position = vr;
                     if (CheckBox3->Checked)
                        LogData(Tact,vr);

                     Yt_ringbuff[Yt_in] = vr;
                     Yt_in = (Yt_in + 1)%YTbuflen;
                     YT();
                     int tp = 2500 - TrackBar1->Position;
                     if (vr > tp)
                     {
                         Label6->Caption="rf > trigger";
                         Label6->Color = clLime;
                         if (CheckBox2->Checked)
                            Beep();
                     }
                     else
                     {
                         Label6->Caption="rf < trigger";
                         Label6->Color = clRed;
                     }
                  }
                  Memo1->Lines->Add(data); // Memo
                  state = 0; // everything again ...
              }
            }
            break;
        } // end case 1
    } // end switch
}
//---------------------------------------------------------------------------
void __fastcall TForm3::FormClose(TObject *Sender, TCloseAction &Action)
{
    if (disp)
    {
       RX_SerialThread->Send("fs00\r");  // switch default
       delete [] Yt_ringbuff;
       Timer1->Enabled = false; // Stop
    }
    disp = false;
}
//---------------------------------------------------------------------------
void __fastcall TForm3::YT(void)
{
    // das Feld fr Image1 muss >> sein (z.B. 1200x1600)
    // als das aktuelle Fenster (y.B. 500x300)
    // damit ausreichend gezoomt werden kann.
    char st[80];
    TCanvas *pPlott = Image1->Canvas;
    pPlott->Pen->Color=clBlack;

    // Ausserer Rahmen (Bildflaeche)
    int ALeft=0;
    int ATop=0;
    int ARight=500;
    int ABottom=320;

    //ClientWidth=Image1->Width;
    //ClientHeight=Image1->Height;
    
    TRect Rahmen = Rect(ALeft,ATop,ARight, ABottom); // ALeft, ATop,  ARight,  ABottom
    pPlott->Brush->Color=clWhite;//clYellow;
    pPlott->FillRect(Rahmen);

    // Innerer (Plott)Rahmen
    int BLeft=ALeft+38; int BTop=ATop+25; int BRight=ARight-50; int BBottom=ABottom-50;
    TRect Plottarea = Rect(BLeft,BTop,BRight, BBottom); // ALeft, ATop,  ARight,  ABottom
    pPlott->Brush->Color=clNavy;//clYellow;clNavy;//clBlack;
    pPlott->FillRect(Plottarea);
    pPlott->FrameRect(Plottarea);

    int Ny=10;  // number of zwischenrume
    int Dy=250; // value between 2 ticks
    float Deltay = 2500.0;
    float miny = 0.0;
    float maxy = 2500.0;
    if (CheckBox1->Checked)
    {
       miny = 2500.0;
       maxy = 0.0;
       for (int i=0; i< Yt_in; i++)
       {
          if (Yt_ringbuff[i] > maxy) maxy = Yt_ringbuff[i];
          if (Yt_ringbuff[i] < miny) miny = Yt_ringbuff[i];
       }
       Deltay = maxy-miny;
       while (Deltay < 200.0)
       {
          maxy++;
          miny--;
          Deltay = maxy-miny;
       }
       maxy=maxy+20.0;
       miny=miny-20.0;
       if (maxy > 2500.0) maxy = 2500.0;
       if (miny <    0.0) miny =    0.0;
       Deltay = maxy-miny;
       Dy = Deltay/Ny;
    }

    // y-ticks and y-label
    char Label[80];
    pPlott->Font->Size=10;
    pPlott->Brush->Color=clWhite; // Schriftfarbe
    pPlott->Pen->Width=1; // tickbreite
    float ygain=(float)(BBottom-BTop)/(float)Ny; // pixels between 2 ticks
    int i=0;
    int x1=BLeft; int x2=x1-5;
    float px, py;
    int w, h;
    do {
        py=(float)BTop+(float)i*ygain;
        pPlott->MoveTo(x1,py);
        pPlott->LineTo(x2,py);
        sprintf(Label,"%5u",int(miny)+(Ny-i)*Dy); // y-label
        w=pPlott->TextWidth(Label);
        h=pPlott->TextHeight(Label);
        pPlott->TextOut(BLeft-6-w,py-h/2,Label);
        i++;
    } while (i<=Ny);
    sprintf(Label,"Signal [mV] at %7.3f MHz",fnom); // y-label
    pPlott->TextOut(ALeft+2,ATop+2,Label);

    // x-ticks
    long Nx=YTbuflen;
    float xgain=(float)(BRight-BLeft)/(float)Nx;
    i=0;
    int y1=BBottom; int y2=y1+5; int y3=y2+4;
    do {
        px=(float)BLeft+(float)i*xgain;
        if ((i%(Nx/50))==0)
        {
            pPlott->MoveTo(px,y1);
            pPlott->LineTo(px,y2);
        }
        if ((i%(Nx/10))==0)
        {
            sprintf(Label,"%u",i); // x-label
            w=pPlott->TextWidth(Label);
            h=pPlott->TextHeight(Label);
            pPlott->LineTo(px,y3);
            pPlott->TextOut(px-w/2,y2+h/2,Label);
        }
        i++;
    } while (i<=Nx);
    TDateTime dtPresent = Now();
    //DecodeDate(dtPresent, Year, Month, Day);
    AnsiString DatumSt=DateToStr(dtPresent);
    //DecodeTime(dtPresent, Hour, Min, Sec, MSec);
    AnsiString TimerSt=TimeToStr(Time()); // needed for logfiles
    //sprintf(Label,"Time stamp [x %Gsec] at %sUT %s",deltaT,TimerSt.c_str(),DatumSt.c_str()); // y-label
    sprintf(Label,"Time stamp"); // y-label
    w=pPlott->TextWidth(Label);
    h=pPlott->TextHeight(Label);
    pPlott->TextOut((BLeft+BRight)/2-w/2,y3+h+2,Label);

    // title
    sprintf(Label,"Intensity in time domain");
    pPlott->Font->Size=11;
    w=pPlott->TextWidth(Label);
    pPlott->TextOut(ARight-w-100,ATop+2,Label); // right aligned
    //sprintf(st,"Yt  %s","manual");
    //Form3->Caption=st;

    // function-plott
    ygain=(float)(BBottom-BTop)/Deltay;
    xgain=(float)(BRight-BLeft)/((float)Nx+1);
    float yA;
    long Yt_temp, p1=0;

    if (Yt_in>0) Yt_temp=Yt_ringbuff[Yt_in-1];
    do
    {
        Yt_temp = (Yt_in+p1)%YTbuflen;
        yA = Yt_ringbuff[Yt_temp]-miny;

       // px = BLeft+((float)p1*xgain);
        px = BRight-((float)p1*xgain);
        py = BBottom-ygain*yA;
        if (py<BBottom) pPlott->Pixels[px][py] = clLime;
    } while(p1++ < Nx);
}
//---------------------------------------------------------------------------
void __fastcall TForm3::Button1Click(TObject *Sender)
{
     for (int i=0; i< YTbuflen; i++)
     {
        Yt_ringbuff[i]=0.0;
     }
     Yt_in = 0;
}
//---------------------------------------------------------------------------
void __fastcall TForm3::LogData(float time, float voltage)
{
    FILE *out;

    try
    {
        if (!FileExists(filename))logfirst = true;
        out = fopen(filename,"a"); // open for append
        if (logfirst)
        {
            logfirst = false;
            char st1[40]="PC-Time (UT) [HH.hhhhh]";
            char st2[40]="Amplitude [mV]";

            if (strstr(delimiter,"TAB")) fprintf(out,"%c%s%c%c%c%s%c\n",34,st1,34,  9,34,st2,34);
            else
            if (strstr(delimiter,","))   fprintf(out,"%c%s%c%c%c%s%c\n",34,st1,34,',',34,st2,34);
            else
            if (strstr(delimiter,";"))   fprintf(out,"%c%s%c%c%c%s%c\n",34,st1,34,';',34,st2,34);

        }
        if (strstr(delimiter,"TAB")) fprintf(out,"%9.5f%c%9.4f\n",time,  9,voltage);
        else
        if (strstr(delimiter,","))   fprintf(out,"%9.5f%c%9.4f\n",time,',',voltage);
        else
        if (strstr(delimiter,";"))   fprintf(out,"%9.5f%c%9.4f\n",time,';',voltage);

        fclose (out);
    }
    catch (Exception &exception)
    {
        Label13->Visible = true;
        Label13->Caption = "Error logfile";
    }
}
//---------------------------------------------------------------------------
void __fastcall TForm3::Button2Click(TObject *Sender) // close
{
    if (disp)
    {
       RX_SerialThread->Send("fs00\r");  // switch default
       delete [] Yt_ringbuff;
       Timer1->Enabled = false; // Stop
    }
    disp = false;

    Form3->Close();
}
//---------------------------------------------------------------------------
void __fastcall TForm3::Button3Click(TObject *Sender) // start
{
    Timer1->Enabled = true;  // Start
}
//---------------------------------------------------------------------------
void __fastcall TForm3::Button4Click(TObject *Sender) // stop
{
    Timer1->Enabled = false;  // Stop
}
//---------------------------------------------------------------------------
void __fastcall TForm3::Button5Click(TObject *Sender)
{
   char st[40];
   float frequency = atof(Edit1->Text.c_str());
   if ((frequency >=  45.0) && (frequency <= 870.0))
   {
        sprintf(st,"F0%7.3f\r",frequency); // frequency control
        RX_SerialThread->Send(st);
   } else
   {
      ShowMessage("Frequency out of range");
      return;
   }

   pwm = atoi(Edit2->Text.c_str());
   if ((pwm >= 1) && (pwm <= 255))
   {
      sprintf(st,"O%03u\r",pwm); // gain control
      RX_SerialThread->Send(st);
   } else
   {
      ShowMessage("PWM out of range");
      return;
   }

   focus = atoi(Edit3->Text.c_str());
   if ((focus >= 0) && (focus <= 63))
   {
      sprintf(st,"fs%02i\r",focus);
      RX_SerialThread->Send(st);  // switch cold
   } else
   {
      ShowMessage("Focuscode out of range");
      return;
   }

   int t = atoi(Edit4->Text.c_str()); // msec
   if ((t >= 50) && (t <= 20000))
   {
      sampling = t;
      NN = sampling/(2*Timer1->Interval); // measurements
   } else
   {
      ShowMessage("Samplingtime out of range");
      return;
   }
}
//---------------------------------------------------------------------------

