/*-----------------------------------------------------------------------*/
/*  (C) Copyright Institute of Astronomy ETHZ 8092 Zuerich Switzerland   */
/*-----------------------------------------------------------------------*/
/*  Programmname: rxrs232.cpp                                            */
/*                                                                       */
/*  Revision: V1.0     Date: 05.05.2003    Autor: Chr. Monstein          */
/*                                                                       */
/*  Purpose: Receiver control unit Radiospectrometer CALLISTO            */
/*                                                                       */
/*  Compiler: BORLAND C++Builder 6                                       */
/*-----------------------------------------------------------------------*/

// Created by: Chr. Monstein 05.05.2003
// Updated by: Chr. Monstein 17.06.2003
// Updated by: Chr. Monstein 26.11.2004 programmable baudrate, integration deleted
// Updated by: Chr. Monstein 13.09.2012 programmable baudrate, integration deleted
// Updated by: Chr. Monstein 19.12.2012 clean up

//---------------------------------------------------------------------------
#include <vcl.h>
#pragma hdrstop

#define WIN32

#include <stdio.h> // sprintf

#include "WSC.H"
#include "RXRS232.h"
#include "mainunit.h"

#pragma package(smart_init)
//---------------------------------------------------------------------------

TWrite *RX_SerialThread;

int RX_Port; // Receiver control unit RCU
int RX_Baud = WSC_Baud115200; //   static...

Byte RX_ringbuff[RXbuflen];
long RX_in = 0, RX_out = 0;
//---------------------------------------------------------------------------
extern int rows;             // import from mainunit.cpp, number of measurement points per sweep
extern int nsps;             // import from mainunit.cpp, number of sweeps per second
extern int observationmode;  // import from mainunit.cpp, 0..8
extern int clocksource;      // import from mainunit.cpp, RISC-timer source
extern int divider;          // import from mainunit.cpp, factor to convert ADC -> FITS-file
int comstatus=-1;
//---------------------------------------------------------------------------
__fastcall TWrite::TWrite(bool CreateSuspended) : TThread(CreateSuspended)
{
}
//---------------------------------------------------------------------------
void __fastcall TWrite::Init(int comportnumber)
{
    char Temp[80];

    RX_Port = comportnumber-1; // evtl -1 ????

    // Code = SioDebug('R'); // power USB-device
    comstatus = SioReset(RX_Port,8192,1024); // COM, RX-buffer, TX-buffer
    if(comstatus<0)
    {
        SioWinError((LPSTR)Temp,80);
        ieee->Label4->Caption = Temp;
        ieee->Label4->Caption = "Can't open RCU-port";
    }
    else
    {
        SioParms(RX_Port,WSC_NoParity,WSC_OneStopBit,WSC_WordLength8);
        SioBaud(RX_Port,RX_Baud);
        SioDTR(RX_Port,'S');
        SioRTS(RX_Port,'S');
        ieee->Label4->Caption = "Port available";
    }
    ieee->Update();
}
//---------------------------------------------------------------------------
void __fastcall TWrite::Transmit(char byte) // Send byte to client via RS
{
    SioPutc(RX_Port, byte);
}
//---------------------------------------------------------------------------
void __fastcall TWrite::Send(char * cmd) // Send byte to client via RS
{
    SioPuts(RX_Port, (LPSTR)cmd, strlen(cmd)); // string to receiver
}
//---------------------------------------------------------------------------
void __fastcall TWrite::GetRXRS232Data(void) // Private activities
{
    int cnt;
    char buffer[512]; // we expect 4KByte/sec = 80Byte/20msec
    cnt = SioGets(RX_Port,(LPSTR)buffer,512);  // (LPSTR) = mit null beendeter String
    for (int i=0; i<cnt; i++) // fill up FPU ringbuffer
    {
        RX_ringbuff[RX_in] = buffer[i];
        RX_in = (RX_in+1)%RXbuflen;
    }
}
//---------------------------------------------------------------------------
void __fastcall TWrite::Execute() // Main job extra-thread
{
    while(1) // almost forever...
    {
        if(Terminated) return; // exit thread
        GetRXRS232Data();
        Sleep(2); // wait  ???
    }
}
//---------------------------------------------------------------------------
void __fastcall TWrite::Close() // termination
{
    //Send("S0\r"); // stop state machine
    SioDone(RX_Port);
    Terminate(); //RX_SerialThread->Terminate();
    WaitFor(); //RX_SerialThread->WaitFor();
}
//---------------------------------------------------------------------------
