/*-----------------------------------------------------------------------*/
/*  (C) Copyright Institute of Astronomy ETHZ 8092 Zuerich Switzerland   */
/*-----------------------------------------------------------------------*/
/*  Programmname: mainunit.cpp                                           */
/*                                                                       */
/*  Revision: V1.0     Date: 05.05.2003    Autor: Chr. Monstein          */
/*                                                                       */
/*  Purpose: Noise Figure Measurement of a CALLISTO                      */
/*                                                                       */
/*  Compiler: BORLAND C++Builder 6                                       */
/*-----------------------------------------------------------------------*/

// Created by: Chr. Monstein 05.05.2003
// Updated by: Chr. Monstein 17.06.2003
// Updated by: Chr. Monstein 26.11.2004 programmable baudrate, integration deleted
// Updated by: Chr. Monstein 29.09.2012 improvements
// Updated by: Chr. Monstein 19.12.2012 memory management improved (pointer)
// Updated by: Chr. Monstein 25.12.2012 new functions
// Updated by: Chr. Monstein 01.01.2013 new functions v1.5
// Updated by: Chr. Monstein 09.01.2013 new functions v1.8 multiple import of arrays deleted
// Updated by: Chr. Monstein 29.02.2013 new functions v1.9 AGC lineariy

//---------------------------------------------------------------------------

#pragma hdrstop
#include <vcl.h>

#include <math.h>
#include <fstream.h>
#include <stdio.h>
#include <windows.h>

#include "mainunit.h"
#include "WSC.H"
#include "RXRS232.H"
#include "PlottUnit.h"
#include "MyGlobal.h"
#include "FCunit.h"
#include "VoltageUnit.h"
#include "DigitizerUnit.h"

//---------------------------------------------------------------------------
#pragma package(smart_init)
#pragma resource "*.dfm"

//---------------------------------------------------------------------------

TPlotter *MyPlott;

int anzMess = 0;  //zaehlt messwerte
Tieee *ieee;

int integration;
int Nr_channels;
int mode; // cold/warm/hot
int processstate = 0;

float fmin, fnom, fmax;
float f, df;
float ENR;
int pwm;                   // gain control
int focus;                 // focuscode for digitizer/scope
int sampling;              // sampling time digitizer/scope
float gradient;            // slope mv/dB
float xarr_all[MaxRows];   // muss static sein
float yarr_cold[MaxRows];  // 1
float yarr_warm[MaxRows];  // 2
float yarr_hot [MaxRows];  // 3
float grad [MaxRows];      //
float nfdb [MaxRows];      //
float bpf  [MaxRows];      //
Tswitch MyRelais; // structure contains switching information for calibration
int errorcount;

// import from digitizer.cpp
extern char logpath[240];         // digitizer log
extern char ending[10];           // digitizer file-ending
extern char delimiter[5];         // digitizer deli

int xleft, ytop, xsize,ysize; // plot window
FILE *LogFileID;      // to save all receiver, fpu and host messages
char logfilename[80]; // actual (last) logfile

// import form RXRS232.cpp
bool connected = false;
bool stopped1  = false;
bool stopped2  = false;
extern TWrite *RX_SerialThread; // RX activities
extern Byte RX_ringbuff[RXbuflen];
extern long RX_in, RX_out;
extern int comstatus;

//---------------------------------------------------------------------------
void __fastcall Tieee::ReadConfig(void)
{
  char buff[120];
  AnsiString st;
  int i = 0;

  ifstream infile("nf.cfg");
  while (infile)
  {
      infile.getline(buff,120);
      st = LowerCase(AnsiString(buff));
      if (buff[0]=='[')
      {
         // measuring parameter
         if (strstr(st.c_str(),"[rxcomport]="))    {sscanf(buff,"[rxcomport]=%s"   ,&buff); Edit1->Text = buff;  i++;}
         if (strstr(st.c_str(),"[enr]="))          {sscanf(buff,"[enr]=%s"         ,&buff); Edit12->Text = buff; i++;}
         if (strstr(st.c_str(),"[detector]="))     {sscanf(buff,"[detector]=%s"    ,&buff); Edit17->Text = buff; i++;}
         if (strstr(st.c_str(),"[channels]="))     {sscanf(buff,"[channels]=%s"    ,&buff); Edit2->Text = buff;  i++;}
         if (strstr(st.c_str(),"[integrations]=")) {sscanf(buff,"[integrations]=%s",&buff); Edit16->Text = buff; i++;}
         if (strstr(st.c_str(),"[fmin]="))         {sscanf(buff,"[fmin]=%s"        ,&buff); Edit5->Text = buff;  i++;}
         if (strstr(st.c_str(),"[fnom]="))         {sscanf(buff,"[fnom]=%f"        ,&fnom);                      i++;}
         if (strstr(st.c_str(),"[fmax]="))         {sscanf(buff,"[fmax]=%s"        ,&buff); Edit4->Text = buff;  i++;}
         if (strstr(st.c_str(),"[pwm]="))          {sscanf(buff,"[pwm]=%s"         ,&buff); Edit3->Text = buff;  i++;}
         if (strstr(st.c_str(),"[maintitle]="))    {sscanf(buff,"[maintitle]=%s"   ,&buff); Edit11->Text = buff; i++;}
         if (strstr(st.c_str(),"[sstitle]="))      {sscanf(buff,"[sstitle]=%s"     ,&buff); Edit19->Text = buff; i++;}

         // window paramter
         if (strstr(st.c_str(),"[xleft]="))        {sscanf(buff,"[xleft]=%i"       ,&xleft);  i++;}
         if (strstr(st.c_str(),"[ytop]="))         {sscanf(buff,"[ytop]=%i"        ,&ytop);   i++;}
         if (strstr(st.c_str(),"[xsize]="))        {sscanf(buff,"[xsize]=%i"       ,&xsize);  i++;}
         if (strstr(st.c_str(),"[ysize]="))        {sscanf(buff,"[ysize]=%i"       ,&ysize);  i++;}

         // plott parameter
         if (strstr(st.c_str(),"[ylow]="))         {sscanf(buff,"[ylow]=%s"        ,&buff); Edit14->Text = buff; i++;}
         if (strstr(st.c_str(),"[yhigh]="))        {sscanf(buff,"[yhigh]=%s"       ,&buff); Edit13->Text = buff; i++;}
         if (strstr(st.c_str(),"[linecolor]="))    {sscanf(buff,"[linecolor]=%s"   ,&buff); Edit9->Text = buff;  i++;}
         if (strstr(st.c_str(),"[backcolor]="))    {sscanf(buff,"[backcolor]=%s"   ,&buff); Edit18->Text = buff; i++;}
         if (strstr(st.c_str(),"[yticks]="))       {sscanf(buff,"[yticks]=%s"      ,&buff); Edit6->Text = buff;  i++;}
         if (strstr(st.c_str(),"[xticks]="))       {sscanf(buff,"[xticks]=%s"      ,&buff); Edit15->Text = buff; i++;}
         if (strstr(st.c_str(),"[marker]="))       {sscanf(buff,"[marker]=%s"      ,&buff); Edit8->Text = buff;  i++;}
         if (strstr(st.c_str(),"[thickcurve]="))   {sscanf(buff,"[thickcurve]=%s"  ,&buff); Edit7->Text = buff;  i++;}
         if (strstr(st.c_str(),"[thickmarker]="))  {sscanf(buff,"[thickmarker]=%s" ,&buff); Edit10->Text = buff; i++;}

         if (strstr(st.c_str(),"[cold]="))         {sscanf(buff,"[cold]=%02u,%c"   ,&MyRelais.switch_cold,&MyRelais.type_cold); i++;}
         if (strstr(st.c_str(),"[warm]="))         {sscanf(buff,"[warm]=%02u,%c"   ,&MyRelais.switch_warm,&MyRelais.type_warm); i++;}
         if (strstr(st.c_str(),"[hot]="))          {sscanf(buff,"[hot]=%02u,%c"    ,&MyRelais.switch_hot ,&MyRelais.type_hot);  i++;}
         if (strstr(st.c_str(),"[att]="))          {sscanf(buff,"[att]=%f"         ,&MyRelais.attenuator);                      i++;}

         if (strstr(st.c_str(),"[focus]="))        {sscanf(buff,"[focus]=%i"       ,&focus);     i++;}
         if (strstr(st.c_str(),"[sampling]="))     {sscanf(buff,"[sampling]=%i"    ,&sampling);  i++;}
         if (strstr(st.c_str(),"[logpath]="))      {sscanf(buff,"[logpath]=%s"     ,&logpath);   i++;}
         if (strstr(st.c_str(),"[ending]="))       {sscanf(buff,"[ending]=%s"      ,&ending);    i++;}
         if (strstr(st.c_str(),"[delimiter]="))    {sscanf(buff,"[delimiter]=%s"   ,&delimiter); i++;}

      }
  }
  infile.close();

  if (i!=33) // check number of parameter
  {
    Application->MessageBox("Check nf.cfg", "Not sufficient paramter", MB_OK);
  }

  MyPlott->SetWindowCoordinates(xsize,ysize,xleft,ytop);
  pwm = atoi(Edit3->Text.c_str());
  sprintf(buff,"%5.1f",MyRelais.attenuator); Edit20->Text = buff;

}
//---------------------------------------------------------------------------
__fastcall Tieee::Tieee(TComponent* Owner) : TForm(Owner)
{
    MyPlott = new TPlotter;
    MyPlott->Init();

    ReadConfig();

    Button1->Enabled = false; // cold
    Button2->Enabled = false; // hot
    Button4->Enabled = false; // plot NF
    Button5->Enabled = false; // disconnect
    Button6->Enabled = false; // warm
    Button7->Enabled = false; // plot mV/dB
    Button8->Enabled = false; // plot raw
    Digitizer1->Enabled = false;
    AGCtsting1->Enabled = false;
    AGCLIN1->Enabled    = false;
    Voltages1->Enabled  = false;
    Focus1->Enabled=false;
    //Focus1->Enabled = false; // focus
 }
//---------------------------------------------------------------------------
void __fastcall Tieee::Erase() // erase measurement buffer
{
    for (int i=0; i<anzMess; i++)
    {
       yarr_cold[i] = 0;
       yarr_warm[i] = 0;
       yarr_hot[i]  = 0;
    }
}
//---------------------------------------------------------------------------
void __fastcall Tieee::Reset() // erase Rs-232 buffer
{
   for (int i=0; i<RXbuflen; i++)
   {
      RX_ringbuff[i] = 0;
   }
   RX_out = 0;
   RX_in  = 0;
}
//---------------------------------------------------------------------------
void __fastcall Tieee::Button3Click(TObject *Sender) // connect
{
    char st[40];
    short status;

    RX_SerialThread = new TWrite(false);
    Sleep(50);

    sprintf(st,"%s",Edit1->Text.c_str());
    RX_SerialThread->Init(atoi(st)); // Connect RX-interface Port
    Sleep(50);

    RX_SerialThread->Send("%5\r"); // MHz,mV

    Button1->Enabled = true; // cold
    Button2->Enabled = true; // hot
    Button3->Enabled = false; // connect
    Button5->Enabled = true; // disconnect
    Button6->Enabled = true; // warm
    Button9->Enabled = true; // auto

    Digitizer1->Enabled = true;
    AGCtsting1->Enabled = true;
    AGCLIN1->Enabled    = true;
    Voltages1->Enabled  = true;
    Focus1->Enabled     = true;

    connected = true;
}
//---------------------------------------------------------------------------
void __fastcall Tieee::Button5Click(TObject *Sender)  // disconnect
{
    char st[40];
    sprintf(st,"fs%02i\r",MyRelais.switch_cold);
    RX_SerialThread->Send(st);  // switch cold

    RX_SerialThread->Close(); // Connect RX-interface Port
    delete RX_SerialThread;

    //Button1->Enabled = false; // cold
    //Button2->Enabled = false; // hot
    Button3->Enabled = true;  // connect
    Button4->Enabled = false; // plot NF
    Button5->Enabled = false; // disconnect
    //Button6->Enabled = false; // warm
    Button7->Enabled = false; // plot mV/dB
    connected = false;
}
//---------------------------------------------------------------------------
void __fastcall Tieee::Button1Click(TObject *Sender) // start cold
{
    char st[40];
    processstate = 0;
    Form1->Close();
    Form2->Close();
    Form3->Close();
    Reset();
    RX_SerialThread->Send("%5\r"); // MHz,mV

    sprintf(st,"%s",Edit2->Text.c_str());
    Nr_channels = atof(st);
    if ((Nr_channels <1) || (Nr_channels > 13200))
    {
       ShowMessage("Number of channels out of range");
       return;
    }

    sprintf(st,"%s",Edit16->Text.c_str());
    integration = atoi(st);
    if ((integration <1) || (integration > 1000))
    {
       ShowMessage("Number of integrations out of range");
       return;
    }

    sprintf(st,"%s",Edit4->Text.c_str());
    fmax = atof(st);

    sprintf(st,"%s",Edit5->Text.c_str());
    fmin = atof(st);
    f = fmin;
    df = (fmax - fmin)/(float)Nr_channels;

    if ((fmin<45.0) || (fmin>870.0) || (fmax<45.0) || (fmax>870.0) || (fmin>fmax))
    {
       ShowMessage("Frequencies out of range");
       return;
    }

    sprintf(st,"fs%02i\r",MyRelais.switch_cold);
    RX_SerialThread->Send(st);  // switch cold

    sprintf(st,"F0%6.3f\r",fmin);
    RX_SerialThread->Send(st); // set freqeuncy

    pwm = atoi(Edit3->Text.c_str());
    if ((pwm <1) || (pwm > 255))
    {
       ShowMessage("PWM value out of range");
       return;
    }
    sprintf(st,"O%03i\r",pwm); // set gain
    RX_SerialThread->Send(st);

    RX_in = 0;
    RX_out = 0;

    f = fmin;
    anzMess = 0;
    //Button2->Enabled = false; // hot
    Button4->Enabled = false; // plot NF
    //Button6->Enabled = false; // warm
    Button7->Enabled = false; // plot mV/dB
    //Application->MessageBox("connect 50 ohm or switch noise source OFF", "Measure cold load", MB_OK);

    mode = 0;
    errorcount = 0;
    Timer1->Enabled = true;  // Start
}
//---------------------------------------------------------------------------
void __fastcall Tieee::Button6Click(TObject *Sender) // start warm
{
    char st[40];
    f = fmin;
    anzMess = 0;
    Form1->Close();
    Form2->Close();
    Form3->Close();
    Reset();
    RX_SerialThread->Send("%5\r"); // MHz,mV
    //Application->MessageBox("dis-connect 50 ohm or switch noise source ON", "Measure warm load", MB_OK);
    mode = 1;
    sprintf(st,"%s",Edit2->Text.c_str());
    Nr_channels = atof(st);
    if ((Nr_channels <1) || (Nr_channels > 13200))
    {
       ShowMessage("Number of channels out of range");
       return;
    }

    sprintf(st,"%s",Edit16->Text.c_str());
    integration = atoi(st);
    if ((integration <1) || (integration > 1000))
    {
       ShowMessage("Number of integrations out of range");
       return;
    }

    sprintf(st,"%s",Edit4->Text.c_str());
    fmax = atof(st);

    sprintf(st,"%s",Edit5->Text.c_str());
    fmin = atof(st);
    f = fmin;
    df = (fmax - fmin)/(float)Nr_channels;

    if ((fmin<45.0) || (fmin>870.0) || (fmax<45.0) || (fmax>870.0) || (fmin>fmax))
    {
       ShowMessage("Frequencies out of range");
       return;
    }

    sprintf(st,"fs%02i\r",MyRelais.switch_warm);
    RX_SerialThread->Send(st);  // switch 

    sprintf(st,"F0%6.3f\r",fmin);
    RX_SerialThread->Send(st); // set freqeuncy

    pwm = atoi(Edit3->Text.c_str());
    if ((pwm <1) || (pwm > 255))
    {
       ShowMessage("PWM value out of range");
       return;
    }
    sprintf(st,"O%03i\r",pwm); // set gain
    RX_SerialThread->Send(st);

    errorcount = 0;
    Timer1->Enabled = true;  // Start
}
//---------------------------------------------------------------------------
void __fastcall Tieee::Button2Click(TObject *Sender) // start hot
{
    char st[40];
    f = fmin;
    anzMess = 0;
    Form1->Close();
    Form2->Close();
    Form3->Close();
    Reset();
    RX_SerialThread->Send("%5\r"); // MHz,mV
    //Application->MessageBox("dis-connect 50 ohm or switch noise source ON", "Measure hot load", MB_OK);
    mode = 2;

    sprintf(st,"%s",Edit2->Text.c_str());
    Nr_channels = atof(st);
    if ((Nr_channels <1) || (Nr_channels > 13200))
    {
       ShowMessage("Number of channels out of range");
       return;
    }

    sprintf(st,"%s",Edit16->Text.c_str());
    integration = atoi(st);
    if ((integration <1) || (integration > 1000))
    {
       ShowMessage("Number of integrations out of range");
       return;
    }

    sprintf(st,"%s",Edit4->Text.c_str());
    fmax = atof(st);

    sprintf(st,"%s",Edit5->Text.c_str());
    fmin = atof(st);
    f = fmin;
    df = (fmax - fmin)/(float)Nr_channels;

    if ((fmin<45.0) || (fmin>870.0) || (fmax<45.0) || (fmax>870.0) || (fmin>fmax))
    {
       ShowMessage("Frequencies out of range");
       return;
    }

    sprintf(st,"fs%02i\r",MyRelais.switch_hot);
    RX_SerialThread->Send(st);  // switch

    sprintf(st,"F0%6.3f\r",fmin);
    RX_SerialThread->Send(st); // set freqeuncy

    pwm = atoi(Edit3->Text.c_str());
    if ((pwm <1) || (pwm > 255))
    {
       ShowMessage("PWM value out of range");
       return;
    }
    sprintf(st,"O%03i\r",pwm); // set gain
    RX_SerialThread->Send(st);

    errorcount = 0;
    Timer1->Enabled = true;  // Start
}
//---------------------------------------------------------------------------
__fastcall Tieee::~Tieee()   // Destructor
{

}
//---------------------------------------------------------------------------
void __fastcall Tieee::Timer1Timer(TObject *Sender)  // Ablauf
{
    short status;
    char st[80], data[RXbuflen];
    #define preamble -100
    static int state = preamble;
    static int j = 0;
    char ch;
    static float f1;
    static int v1;
    static int integration_result;
    static int integration_count;
    if (stopped1)
    {
       state = preamble;
       j = 0;
       stopped1 = false;
    }

    switch(state) // execute every x msec, where x<10
    {
        default: // wait 100*5msec (relay-switch-time)
        {
            RX_SerialThread->Send("%5\r"); // MHz,mV

            if (((j++)%10)==0)
                  Label31->Caption='/';
            else
                  Label31->Caption="\\";

            RX_SerialThread->Send("A0\r");
            f = fmin;
            ProgressBar1->Min=0;
            ProgressBar1->Max=Nr_channels;
            ProgressBar1->Min=0;
            int i = 0;
            do
            {
                ch = RX_ringbuff[RX_out];
                RX_out = (RX_out+1)%RXbuflen;
                if (ch==13) break;
                if (i>(RXbuflen/4)) break; // emergency exit
                data[i++] = ch;
            } while (1);
            data[i]='\0';
            Label30->Caption = "Errors";
            state++;
            break;
        }

        case 0: // set frequency
        {
            sprintf(st,"F0%6.3f\r",f);
            RX_SerialThread->Send(st);
            Label31->Caption='-';
            ProgressBar1->Position = anzMess;
            integration_count = 0;
            integration_result = 0;
            state++;
            break;
        }

        case 1: // read ADC
        {
            RX_SerialThread->Send("A0\r");
            integration_count++;
            state++;
            break;
        }

        case 2: // wait for incoming data
        {
            int i = 0;
            do
            {
                ch = RX_ringbuff[RX_out];
                RX_out = (RX_out+1)%RXbuflen;
                if (ch==13) break;
                if (i>(RXbuflen/4)) break; // emergeny exit
                data[i++] = ch;
            } while (1);
            data[i] = '\0';

            int p = sscanf(data,"$CRX:%f,%u",&f1,&v1);
            if ((p==2) && (v1 > 200.0) && (v1 < 2500.0))
            {
                Label5->Caption = data; // obviously o.k.
            } else
            {
               state = 1;
               errorcount++; // something wrong...
               sprintf(st,"Errors %i",errorcount);
               Label30->Caption = st;
               break;
            }
            if (integration_count <= integration)
            {
               state = 1;
               integration_result = integration_result + v1;
               break;
            }
            else v1 =  integration_result/(integration);

            sprintf(st,"%6.3f,%u",f,v1);
            xarr_all[anzMess] = f;
            switch(mode)
            {
               case 0:
               {
                  yarr_cold[anzMess] = v1;
                  break;
               }
               case 1:
               {
                  yarr_warm[anzMess] = v1;
                  break;
               }
               case 2:
               {
                  yarr_hot[anzMess] = v1;
                  break;
               }
            }

            v1 = 0;
            f = f + df;
            anzMess = anzMess + 1;
            state = 0; // everything again ...

            if (f > fmax && (mode==0)) // cold
            {
                Timer1->Enabled  = false; // stop
                //Button1->Enabled = false; // cold
                //Button2->Enabled = false; // hot
                Button4->Enabled = false;  // plott
                Button6->Enabled = true;  // warm
                Button7->Enabled = false;  // plott g
                Button8->Enabled = true; // plot mV
                sprintf(st,"F0%6.3f\r",fmin);
                RX_SerialThread->Send(st);
                Button8Click(0); // plot cold mV
                state = preamble;
            } else

            if ((f > fmax) && (mode==1)) // warm
            {
                Timer1->Enabled  = false;  // stop
                //Button1->Enabled = false;  // cold
                //Button2->Enabled = true;   // hot
                Button4->Enabled = false;  // plott
                //Button6->Enabled = false;  // warm
                Button7->Enabled = false;  // plott g
                sprintf(st,"F0%6.3f\r",fmin);
                RX_SerialThread->Send(st);
                Button8Click(0); // plot warm mV
                state = preamble;
            } else

            if ((f > fmax) && (mode==2)) // hot
            {
                Timer1->Enabled  = false; // stop
                //Button1->Enabled = true;  // cold
                //Button2->Enabled = false; // hot
                Button4->Enabled = true;  // plott
                //Button6->Enabled = false; // warm
                Button7->Enabled = true;  // plott g
                sprintf(st,"F0%6.3f\r",fmin);
                RX_SerialThread->Send(st);
                //Button4Click(0);  // plott
                sprintf(st,"fs%02i\r",MyRelais.switch_cold);
                RX_SerialThread->Send(st);  // switch cold
                Button8Click(0); // plot hot mV
                state = preamble;
            }
            break;
        }
    } // end switch
}
//---------------------------------------------------------------------------
void __fastcall Tieee::Button4Click(TObject *Sender) // Plot NF
{
    char Title[bufferwidth], st[bufferwidth], Parameter[bufferwidth],
         subtitle[bufferwidth], file[bufferwidth], extension[5];

    Word Year, Month, Day, Hour, Min, Sec, MSec;

    sprintf(st,"%s",Edit17->Text.c_str());
    gradient = atof(st);
    if (gradient < 1.0) gradient = 1.0;

    sprintf(st,"%s",Edit12->Text.c_str());
    ENR = atof(st);
    if (ENR < 1.0) ENR = 1.0;

    float ydb, ylin;
    MyPlott->SetNumber(anzMess);
    MyPlott->SetXvector(xarr_all);
    float average=0.0;
    for (int i=0; i<anzMess; i++)
    {
       ydb     = abs(yarr_hot[i] - yarr_cold[i])/gradient; // Y-dB
       ylin    = pow(10.0,ydb/10.0);
       nfdb[i] = ENR - 10.0*log10(ylin-0.999);
       average = average + nfdb[i];
    }
    average = average / anzMess;
    MyPlott->SetYvector1(nfdb);

    sprintf(Title,"Noise figure e-Callisto #%s",Edit11->Text.c_str());
    MyPlott->SetTitle(Title);

    sprintf(Parameter,"g=%5.2f mV/dB, pwm=%3i digit, ENR=%5.1f dB",gradient,pwm,ENR);
    MyPlott->SetSubTitle(Parameter);

    float ss=0.0; float sigma;
    for (int i=0; i<anzMess; i++)
    {
       ss = ss + pow((nfdb[i] - average),2);
    }
    sigma = sqrt(1.0/(float)anzMess*ss);

    sprintf(subtitle,"<%s>, Avg.=%4.1f dB, 1s=%4.1f dB",Edit19->Text.c_str(),average,sigma);
    MyPlott->SetSubSubTitle(subtitle);

    int xticks = atoi(Edit15->Text.c_str());
    MyPlott->SetXminmax(fmin, fmax, (fmax-fmin)/xticks);

    MyPlott->SetXLabel("Frequency [MHz]");
    MyPlott->SetYLabel("Noise figure [dB]");

    MyPlott->SetThickness_Curve(atoi(Edit7->Text.c_str()));
    MyPlott->SetMarker(atoi(Edit8->Text.c_str()));
    MyPlott->SetThickness_Marker(atoi(Edit10->Text.c_str()));

    sprintf(st,"%s",Edit9->Text.c_str());
    MyPlott->SetColor1(st);

    MyPlott->SetBGColor(atoi(Edit18->Text.c_str()));

    float ymin = atof(Edit14->Text.c_str());
    float ymax = atof(Edit13->Text.c_str());
    int yticks = atoi(Edit6->Text.c_str());
    MyPlott->SetYminmax(ymin, ymax , (ymax-ymin)/yticks); // lineare skala

    if (RadioButton1->Checked) {MyPlott->SetDevice("XWIN"); sprintf(extension,"%s","nil");}
    if (RadioButton2->Checked) {MyPlott->SetDevice("PDF");  sprintf(extension,"%s","PDF");}
    if (RadioButton3->Checked) {MyPlott->SetDevice("EPS");  sprintf(extension,"%s","EPS");}
    if (RadioButton4->Checked) {MyPlott->SetDevice("TIFF"); sprintf(extension,"%s","TIF");}
    if (RadioButton5->Checked) {MyPlott->SetDevice("GIF");  sprintf(extension,"%s","GIF");}
    if (RadioButton6->Checked) {MyPlott->SetDevice("BMP");  sprintf(extension,"%s","BMP");}
    if (RadioButton7->Checked) {MyPlott->SetDevice("PNG");  sprintf(extension,"%s","PNG");}

    TDateTime dtPresent = Now();
    DecodeDate(dtPresent, Year, Month, Day);

    DecodeTime(dtPresent, Hour, Min, Sec, MSec);

    sprintf(file,"NF_%s_PWM%3i_%04u-%02u-%02u_%02u%02u%02u.%s",
            Edit11->Text.c_str(),pwm,Year, Month, Day,Hour, Min, Sec,extension);
    MyPlott->SetFilename(file);

    MyPlott->LinPlott(true); // single

    Timer2->Enabled=false;
}
//---------------------------------------------------------------------------
void __fastcall Tieee::Button7Click(TObject *Sender)  // Plot mV/dB
{
    char Title[bufferwidth], st[bufferwidth], Parameter[bufferwidth],
         subtitle[bufferwidth], file[bufferwidth], extension[5];
    Word Year, Month, Day, Hour, Min, Sec, MSec;

    float ydb, ylin;
    MyPlott->SetNumber(anzMess);
    MyPlott->SetXvector(xarr_all);

    sprintf(st,"%s",Edit20->Text.c_str());
    float ATT = atof(st);
    if (ATT < 0.0) ATT = 0.0;

    float average = 0.0;
    for (int i=0; i<anzMess; i++)
    {
       grad[i] = abs(yarr_hot[i] - yarr_warm[i])/ATT; // mV/dB
       average = average + grad[i];
    }
    average = average / anzMess;
    MyPlott->SetYvector1(grad);

    sprintf(Title,"Detector e-Callisto #%s",Edit11->Text.c_str());
    MyPlott->SetTitle(Title);

    sprintf(Parameter,"att=%5.2f dB, pwm=%3i digit, ENR=%5.1f dB",ATT,pwm,ENR);
    MyPlott->SetSubTitle(Parameter);

    float ss=0.0; float sigma;
    for (int i=0; i<anzMess; i++)
    {
       ss = ss + pow((grad[i] - average),2);
    }
    sigma = sqrt(1.0/(float)anzMess*ss);

    sprintf(subtitle,"<%s>, Avg.=%4.1f mV/dB, 1s=%3.1f",Edit19->Text.c_str(),average,sigma);
    MyPlott->SetSubSubTitle(subtitle);

    int xticks = atoi(Edit15->Text.c_str());
    MyPlott->SetXminmax(fmin, fmax, (fmax-fmin)/xticks);

    MyPlott->SetXLabel("Frequency [MHz]");
    MyPlott->SetYLabel("Detector [mV/dB]");

    MyPlott->SetThickness_Curve(atoi(Edit7->Text.c_str()));
    MyPlott->SetMarker(atoi(Edit8->Text.c_str()));
    MyPlott->SetThickness_Marker(atoi(Edit10->Text.c_str()));
    MyPlott->SetColor1("blue");

    MyPlott->SetBGColor(atoi(Edit18->Text.c_str()));

    float ymin = 10;
    float ymax = 30;
    int yticks = atoi(Edit6->Text.c_str());
    MyPlott->SetYminmax(ymin, ymax , (ymax-ymin)/yticks); // lineare skala

    if (RadioButton1->Checked) {MyPlott->SetDevice("XWIN"); sprintf(extension,"%s","nil");}
    if (RadioButton2->Checked) {MyPlott->SetDevice("PDF");  sprintf(extension,"%s","PDF");}
    if (RadioButton3->Checked) {MyPlott->SetDevice("EPS");  sprintf(extension,"%s","EPS");}
    if (RadioButton4->Checked) {MyPlott->SetDevice("TIFF"); sprintf(extension,"%s","TIF");}
    if (RadioButton5->Checked) {MyPlott->SetDevice("GIF");  sprintf(extension,"%s","GIF");}
    if (RadioButton6->Checked) {MyPlott->SetDevice("BMP");  sprintf(extension,"%s","BMP");}
    if (RadioButton7->Checked) {MyPlott->SetDevice("PNG");  sprintf(extension,"%s","PNG");}

    TDateTime dtPresent = Now();
    DecodeDate(dtPresent, Year, Month, Day);

    DecodeTime(dtPresent, Hour, Min, Sec, MSec);

    sprintf(file,"DB_%s_PWM%3i_%04u-%02u-%02u_%02u%02u%02u.%s",
            Edit11->Text.c_str(),pwm,Year, Month, Day,Hour, Min, Sec,extension);
    MyPlott->SetFilename(file);

    MyPlott->LinPlott(true); // single
}
//---------------------------------------------------------------------------
void __fastcall Tieee::Button9Click(TObject *Sender)  // auto process
{
    char st[40];
    sprintf(st,"%s",Edit2->Text.c_str());
    Nr_channels = atof(st);
    if ((Nr_channels <1) || (Nr_channels > 13200))
    {
       ShowMessage("Number of channels out of range");
       return;
    }

    sprintf(st,"%s",Edit16->Text.c_str());
    integration = atoi(st);
    if ((integration <1) || (integration > 1000))
    {
       ShowMessage("Number of integrations out of range");
       return;
    }

    sprintf(st,"%s",Edit4->Text.c_str());
    fmax = atof(st);

    sprintf(st,"%s",Edit5->Text.c_str());
    fmin = atof(st);
    f = fmin;
    df = (fmax - fmin)/(float)Nr_channels;

    if ((fmin<45.0) || (fmin>870.0) || (fmax<45.0) || (fmax>870.0) || (fmin>fmax))
    {
       ShowMessage("Frequencies out of range");
       return;
    }

    pwm = atoi(Edit3->Text.c_str());
    if ((pwm <1) || (pwm > 255))
    {
       ShowMessage("PWM value out of range");
       return;
    }
    sprintf(st,"O%03i\r",pwm); // set gain
    RX_SerialThread->Send(st);

    Form1->Close();
    Form2->Close();
    Form3->Close();
    Reset();
    Erase();
    Timer2->Enabled = true;
}
//---------------------------------------------------------------------------
void __fastcall Tieee::Timer2Timer(TObject *Sender) // NF auto state machine
{
   static int state = 0;
   Button1->Enabled = false; // cold
   Button2->Enabled = false; // hot
   Button4->Enabled = false; // plot NF
   Button5->Enabled = false; // disconnect
   Button6->Enabled = false; // warm
   Button7->Enabled = false; // plot mV/dB
   Button8->Enabled = false; // plot raw
   Button9->Enabled = false; // auto nf
   Button12->Enabled = false; // update config

   if (stopped2)
   {
      state = 0;
      stopped2 = false;
   }

   switch (state)
   {
      case 0:
      {
         Button1Click(0); // cold
         state++;
         break;
      }
      case 1:
      {
         if (processstate==1)
            state++;
         break;
      }
      case 2:
      {
         //Button6Click(0); // warm
         //state++;
         state = 4; // ignore warm
         break;
      }
      case 3:
      {
         if (processstate==2)
           state++;
         break;
      }
      case 4:
      {
         Button2Click(0); // hot
         state++;
         break;
      }
      case 5:
      {
         if (processstate==3)
         {
            Timer2->Enabled = false;
            Button4Click(0);  // plott NF
            Button1->Enabled = true; // cold
            Button2->Enabled = true; // hot
            Button4->Enabled = true; // plot NF
            Button5->Enabled = true; // disconnect
            Button6->Enabled = true; // warm
            Button7->Enabled = true; // plot mV/dB
            Button8->Enabled = true; // plot raw
            Button9->Enabled = true; // auto nf
            Button12->Enabled = true; // update config
            state = 0;
         }
         break;
      }
   }
}
//---------------------------------------------------------------------------
void __fastcall Tieee::Button8Click(TObject *Sender)  // plot raw data
{
    char Title[bufferwidth], st[bufferwidth], Parameter[bufferwidth],
         subtitle[bufferwidth], file[bufferwidth], extension[5];

    Word Year, Month, Day, Hour, Min, Sec, MSec;

    MyPlott->SetNumber(anzMess);
    MyPlott->SetXvector(xarr_all);

    switch(mode)
    {
       case 0:
       {
          if (CheckBox1->Checked)
          {
            OpenLogging("_cold");
            sprintf(st,"%cMHz%c;%cmV%c",34,34,34,34);
            Logging(st);
            for (int i=0; i<anzMess; i++)
            {
                sprintf(st,"%8.3f;%8.2f",xarr_all[i],yarr_cold[i]);
                Logging(st);
            }
            CloseLogging();
          }
          processstate = 1;
          break;
       }
       case 1:
       {
          if (CheckBox1->Checked)
          {
             OpenLogging("_warm");
             sprintf(st,"%cMHz%c;%cmV%c",34,34,34,34);
             Logging(st);
             for (int i=0; i<anzMess; i++)
             {
                sprintf(st,"%8.3f;%8.2f",xarr_all[i],yarr_warm[i]);
                Logging(st);
             }
             CloseLogging();
          }
          processstate = 2;
          break;
       }
       case 2:
       {
          if (CheckBox1->Checked)
          {
            OpenLogging("_hot");
            sprintf(st,"%cMHz%c;%cmV%c",34,34,34,34);
            Logging(st);
            for (int i=0; i<anzMess; i++)
            {
                sprintf(st,"%8.3f;%8.2f",xarr_all[i],yarr_hot[i]);
                Logging(st);
            }
            CloseLogging();
          }
          processstate = 3;
          break;
       }
    }

    MyPlott->SetYvector1(yarr_cold);
    MyPlott->SetColor1("blue");

    MyPlott->SetYvector2(yarr_warm);
    MyPlott->SetColor2("green");

    MyPlott->SetYvector3(yarr_hot);
    MyPlott->SetColor3("red");

    sprintf(Title,"Raw data e-Callisto #%s",Edit11->Text.c_str());
    MyPlott->SetTitle(Title);

    sprintf(st,"%s",Edit17->Text.c_str());
    gradient = atof(st);

    sprintf(Parameter,"g=%5.2f mV/dB, pwm=%3i digit",gradient,pwm);
    MyPlott->SetSubTitle(Parameter);

    sprintf(subtitle,"<%s>",Edit19->Text.c_str());
    MyPlott->SetSubSubTitle(subtitle);

    int xticks = atoi(Edit15->Text.c_str());
    MyPlott->SetXminmax(fmin, fmax, (fmax-fmin)/xticks);

    MyPlott->SetXLabel("Frequency [MHz]");
    MyPlott->SetYLabel("Detector [mV]");

    MyPlott->SetThickness_Curve(atoi(Edit7->Text.c_str()));
    MyPlott->SetMarker(atoi(Edit8->Text.c_str()));
    MyPlott->SetThickness_Marker(atoi(Edit10->Text.c_str()));

    MyPlott->SetBGColor(atoi(Edit18->Text.c_str()));

    float ymin = 0.0;
    float ymax = 2500.0;
    int yticks = atoi(Edit6->Text.c_str());
    MyPlott->SetYminmax(ymin, ymax , (ymax-ymin)/yticks); // lineare skala

    if (RadioButton1->Checked) {MyPlott->SetDevice("XWIN"); sprintf(extension,"%s","nil");}
    if (RadioButton2->Checked) {MyPlott->SetDevice("PDF");  sprintf(extension,"%s","PDF");}
    if (RadioButton3->Checked) {MyPlott->SetDevice("EPS");  sprintf(extension,"%s","EPS");}
    if (RadioButton4->Checked) {MyPlott->SetDevice("TIFF"); sprintf(extension,"%s","TIF");}
    if (RadioButton5->Checked) {MyPlott->SetDevice("GIF");  sprintf(extension,"%s","GIF");}
    if (RadioButton6->Checked) {MyPlott->SetDevice("BMP");  sprintf(extension,"%s","BMP");}
    if (RadioButton7->Checked) {MyPlott->SetDevice("PNG");  sprintf(extension,"%s","PNG");}

    TDateTime dtPresent = Now();
    DecodeDate(dtPresent, Year, Month, Day);

    DecodeTime(dtPresent, Hour, Min, Sec, MSec);

    sprintf(file,"DB_%s_PWM%3i_%04u-%02u-%02u_%02u%02u%02u.%s",
            Edit11->Text.c_str(),pwm,Year, Month, Day,Hour, Min, Sec,extension);
    MyPlott->SetFilename(file);

    MyPlott->LinPlott(false); // multi = not single
}
//---------------------------------------------------------------------------
void __fastcall Tieee::Button10Click(TObject *Sender) // EXIT
{
    if (connected)
    {
       RX_SerialThread->Close(); // Connect RX-interface Port
       //delete RX_SerialThread;
    }

    MyPlott->Close();
    delete MyPlott;

    Application->Terminate();
}
//---------------------------------------------------------------------------
void __fastcall Tieee::FormClose(TObject *Sender, TCloseAction &Action)
{
   Button10Click(0);
}
//---------------------------------------------------------------------------
void __fastcall Tieee::OpenLogging(char *st)
{
    Word year, month, day, hh, mm, ss, MSec;

    TDateTime dtPresent = Now();
    DecodeDate(dtPresent, year, month, day);
    DecodeTime(dtPresent, hh, mm, ss, MSec);

    sprintf(logfilename,"%sRAW_%04u%02u%02u_%02u%02u%02u%s.txt",logpath,year,month,day,hh,mm,ss,st);
    LogFileID = fopen(logfilename, "at"); // append logfile in text format
}
//---------------------------------------------------------------------------
void __fastcall Tieee::Logging(char *par)
{
  char buff[240];

  sprintf(buff,"%s\n",par);
  fputs(buff,LogFileID);  // write to logfile
}
//---------------------------------------------------------------------------
void __fastcall Tieee::CloseLogging(void)
{
    fflush(LogFileID);
    fclose(LogFileID);
}
//---------------------------------------------------------------------------
void __fastcall Tieee::Timer3Timer(TObject *Sender)
{
   static int state = 0;
   char st[20];
   switch (state)
   {
      case 0:
      {
         sprintf(st,"fs%02i\r",1+4+16);
         RX_SerialThread->Send(st);  // switch cold
         state++;
         break;
      }
      case 1:
      {
         sprintf(st,"fs%02i\r",2+8+32);
         RX_SerialThread->Send(st);  // switch cold
         state++;
         break;
      }
      case 2:
      {
         sprintf(st,"fs%02i\r",63);
         RX_SerialThread->Send(st);  // switch cold
         state++;
         break;
      }
      case 3:
      {
         sprintf(st,"fs%02i\r",0);
         RX_SerialThread->Send(st);  // switch cold
         state = 0;
         Timer3->Enabled = false;
         break;
      }
   }
}
//---------------------------------------------------------------------------
void __fastcall Tieee::Button12Click(TObject *Sender)
{
   ReadConfig();
}
//---------------------------------------------------------------------------
void __fastcall Tieee::Individual1Click(TObject *Sender)
{
   RX_SerialThread->Send("fs00\r");  // switch cold
}
//---------------------------------------------------------------------------
void __fastcall Tieee::Toggle1Click(TObject *Sender)
{
   RX_SerialThread->Send("fs63\r");  // switch cold
}
//---------------------------------------------------------------------------
void __fastcall Tieee::Sequence1Click(TObject *Sender)
{
   Timer3->Enabled = true;
}
//---------------------------------------------------------------------------
void __fastcall Tieee::Individual2Click(TObject *Sender)
{
   Form2->Close();
   Form3->Close();
   Form1->Show(); // FocusCode
}
//---------------------------------------------------------------------------
void __fastcall Tieee::Voltages1Click(TObject *Sender)
{
   Form1->Close();
   Form3->Close();
   Form2->Timer1->Enabled=true;
   Form2->Show(); // Voltages
}
//---------------------------------------------------------------------------
void __fastcall Tieee::Editconfig1Click(TObject *Sender)
{
   WinExec(("notepad.exe nf.cfg"),SW_SHOW);
}
//---------------------------------------------------------------------------
void __fastcall Tieee::Showversioncontrol1Click(TObject *Sender)
{
   WinExec(("notepad.exe VersionControl.txt"),SW_SHOW);
}
//---------------------------------------------------------------------------
void __fastcall Tieee::Digitizer1Click(TObject *Sender)
{
   Form1->Close();
   Form2->Close();

   Form3->Init();
   Form3->Show(); // Digitizer
}
//---------------------------------------------------------------------------
void __fastcall Tieee::AGCtsting1Click(TObject *Sender)
{
   Timer4->Enabled = true;
}
//---------------------------------------------------------------------------
void __fastcall Tieee::Timer4Timer(TObject *Sender) // AGC low pass filter
{
    char Title[bufferwidth], st[bufferwidth], Parameter[bufferwidth],
         subtitle[bufferwidth], file[bufferwidth], extension[5];

    Word Year, Month, Day, Hour, Min, Sec, MSec;
    static int pwm = 0;
    #define pre -75
    static int state = pre;

    char ch;
    char data[RXbuflen];
    float v1;

   switch (state)
   {
      default: // init
      {
         pwm = 0;
         ProgressBar1->Min = 0;
         ProgressBar1->Max = 255;
         ProgressBar1->Position = pwm;
         sprintf(st,"O%03i\r",255); // set gain
         RX_SerialThread->Send(st);
         RX_in=0;
         RX_out=0;
         state++;
         break;
      }
      case 0: // sawtooth
      {
         sprintf(st,"O%03i\r",pwm); // set gain
         RX_SerialThread->Send(st);
         RX_SerialThread->Send("%2\r");
         ProgressBar1->Position = pwm;
         RX_SerialThread->Send("U2\r");
         int i = 0;
          do
          {
              ch = RX_ringbuff[RX_out];
              RX_out = (RX_out+1)%RXbuflen;
              if (ch==13) break;
              if (i>(RXbuflen/4)) break; // emergeny exit
              data[i++] = ch;
          } while (1);
          data[i] = '\0';

         if (strstr(data,"$CRX:AGC gain ="))
         {
            int p = sscanf(data,"$CRX:AGC gain =%f",&v1);
            if (p==1)
            {
               xarr_all[pwm]  = pwm;
               yarr_cold[pwm] = v1;
               pwm++;
            }
         }

         if (pwm > 255)
            state++;
         break;
      }
      case 1:
      {
          Timer4->Enabled = false;
          MyPlott->SetNumber(255);
          MyPlott->SetXvector(xarr_all);
          MyPlott->SetYvector1(yarr_cold);
          MyPlott->SetColor1("blue");

          sprintf(Title,"Gain control e-Callisto #%s",Edit11->Text.c_str());
          MyPlott->SetTitle(Title);

          sprintf(st,"%s",Edit17->Text.c_str());
          gradient = atof(st);

          sprintf(Parameter,"AGC Low Pass Filter check");
          MyPlott->SetSubTitle(Parameter);

          sprintf(subtitle,"Minimum should be between 25 and 50");
          MyPlott->SetSubSubTitle(subtitle);

          MyPlott->SetXminmax(0, 250, 50);

          MyPlott->SetXLabel("PWM value [digit]");
          MyPlott->SetYLabel("Tuner gain control [V]");

          MyPlott->SetThickness_Curve(atoi(Edit7->Text.c_str()));
          MyPlott->SetMarker(atoi(Edit8->Text.c_str()));
          MyPlott->SetThickness_Marker(atoi(Edit10->Text.c_str()));

          MyPlott->SetBGColor(atoi(Edit18->Text.c_str()));

          float ymin = 1.8;
          float ymax = 3.0;
          int yticks = atoi(Edit6->Text.c_str());
          MyPlott->SetYminmax(ymin, ymax , (ymax-ymin)/yticks); // lineare skala

          if (RadioButton1->Checked) {MyPlott->SetDevice("XWIN"); sprintf(extension,"%s","nil");}
          if (RadioButton2->Checked) {MyPlott->SetDevice("PDF");  sprintf(extension,"%s","PDF");}
          if (RadioButton3->Checked) {MyPlott->SetDevice("EPS");  sprintf(extension,"%s","EPS");}
          if (RadioButton4->Checked) {MyPlott->SetDevice("TIFF"); sprintf(extension,"%s","TIF");}
          if (RadioButton5->Checked) {MyPlott->SetDevice("GIF");  sprintf(extension,"%s","GIF");}
          if (RadioButton6->Checked) {MyPlott->SetDevice("BMP");  sprintf(extension,"%s","BMP");}
          if (RadioButton7->Checked) {MyPlott->SetDevice("PNG");  sprintf(extension,"%s","PNG");}

          TDateTime dtPresent = Now();
          DecodeDate(dtPresent, Year, Month, Day);

          DecodeTime(dtPresent, Hour, Min, Sec, MSec);

          sprintf(file,"AGC_%s_PWM%3i_%04u-%02u-%02u_%02u%02u%02u.%s",
                  Edit11->Text.c_str(),pwm,Year, Month, Day,Hour, Min, Sec,extension);
          MyPlott->SetFilename(file);

          MyPlott->LinPlott(true); // single

         state = pre;
         break;
      }
   }
}
//---------------------------------------------------------------------------
void __fastcall Tieee::Button11Click(TObject *Sender) // Stop
{
    Timer1->Enabled  = false;  // stop ablauf
    Timer2->Enabled  = false;  // stop state machine

    stopped1 = true;
    stopped2 = true;

    Button1->Enabled = true; // cold
    Button2->Enabled = true; // hot
    Button4->Enabled = true; // plot NF
    Button5->Enabled = true; // disconnect
    Button6->Enabled = true; // warm
    Button7->Enabled = true; // plot mV/dB
    Button8->Enabled = true; // plot raw
    Button9->Enabled = true; // auto nf
    Button12->Enabled = true; // update config

    Reset();
    Erase();
}
//---------------------------------------------------------------------------


void __fastcall Tieee::AGCLIN1Click(TObject *Sender)
{
   Timer5->Enabled = true;
}
//---------------------------------------------------------------------------
void __fastcall Tieee::Timer5Timer(TObject *Sender)  // AGC linearity
{
    char Title[bufferwidth], st[bufferwidth], Parameter[bufferwidth],
         subtitle[bufferwidth], file[bufferwidth], extension[5];

    Word Year, Month, Day, Hour, Min, Sec, MSec;
    static int pwm = 1;
    #define pre -75
    static int state = pre;

    char ch;
    char data[RXbuflen];
    float v1;

   switch (state)
   {
      default: // init
      {
         pwm = 1;
         ProgressBar1->Min = 0;
         ProgressBar1->Max = 255;
         ProgressBar1->Position = pwm;
         sprintf(st,"O%03i\r",1); // set gain
         RX_SerialThread->Send(st);
         RX_in=0;
         RX_out=0;
         state++;
         break;
      }
      case 0: // sawtooth
      {
         sprintf(st,"O%03i\r",pwm); // set gain
         RX_SerialThread->Send(st);
         RX_SerialThread->Send("%2\r");
         ProgressBar1->Position = pwm;
         RX_SerialThread->Send("U2\r");
         int i = 0;
          do
          {
              ch = RX_ringbuff[RX_out];
              RX_out = (RX_out+1)%RXbuflen;
              if (ch==13) break;
              if (i>(RXbuflen/4)) break; // emergeny exit
              data[i++] = ch;
          } while (1);
          data[i] = '\0';

         if (strstr(data,"$CRX:AGC gain ="))
         {
            int p = sscanf(data,"$CRX:AGC gain =%f",&v1);
            if (p==1)
            {
               xarr_all[pwm]  = pwm;
               yarr_cold[pwm] = v1;
               pwm++;
            }
         }

         if (pwm > 255)
            state++;
         break;
      }
      case 1:
      {
          Timer5->Enabled = false;
          sprintf(st,"O%03i\r",1); // set gain
          RX_SerialThread->Send(st);

          MyPlott->SetNumber(255); //256
          MyPlott->SetXvector(xarr_all);
          MyPlott->SetYvector1(yarr_cold);
          MyPlott->SetColor1("blue");

          sprintf(Title,"Gain control e-Callisto #%s",Edit11->Text.c_str());
          MyPlott->SetTitle(Title);

          sprintf(st,"%s",Edit17->Text.c_str());
          gradient = atof(st);

          sprintf(Parameter,"AGC linearity check");
          MyPlott->SetSubTitle(Parameter);

          sprintf(subtitle,"Plot should have no gaps/jumps");
          MyPlott->SetSubSubTitle(subtitle);

          MyPlott->SetXminmax(10, 250, 40);

          MyPlott->SetXLabel("PWM value [digit]");
          MyPlott->SetYLabel("Tuner gain control [V]");

          MyPlott->SetThickness_Curve(atoi(Edit7->Text.c_str()));
          MyPlott->SetMarker(atoi(Edit8->Text.c_str()));
          MyPlott->SetThickness_Marker(atoi(Edit10->Text.c_str()));

          MyPlott->SetBGColor(atoi(Edit18->Text.c_str()));

          float ymin = 1.8;
          float ymax = 3.0;
          int yticks = atoi(Edit6->Text.c_str());
          MyPlott->SetYminmax(ymin, ymax , (ymax-ymin)/yticks); // lineare skala

          if (RadioButton1->Checked) {MyPlott->SetDevice("XWIN"); sprintf(extension,"%s","nil");}
          if (RadioButton2->Checked) {MyPlott->SetDevice("PDF");  sprintf(extension,"%s","PDF");}
          if (RadioButton3->Checked) {MyPlott->SetDevice("EPS");  sprintf(extension,"%s","EPS");}
          if (RadioButton4->Checked) {MyPlott->SetDevice("TIFF"); sprintf(extension,"%s","TIF");}
          if (RadioButton5->Checked) {MyPlott->SetDevice("GIF");  sprintf(extension,"%s","GIF");}
          if (RadioButton6->Checked) {MyPlott->SetDevice("BMP");  sprintf(extension,"%s","BMP");}
          if (RadioButton7->Checked) {MyPlott->SetDevice("PNG");  sprintf(extension,"%s","PNG");}

          TDateTime dtPresent = Now();
          DecodeDate(dtPresent, Year, Month, Day);

          DecodeTime(dtPresent, Hour, Min, Sec, MSec);

          sprintf(file,"AGCLIN_%s_PWM%3i_%04u-%02u-%02u_%02u%02u%02u.%s",
                  Edit11->Text.c_str(),pwm,Year, Month, Day,Hour, Min, Sec,extension);
          MyPlott->SetFilename(file);

          MyPlott->LinPlott(true); // single

          state = pre;
          break;
      }
   }
}
//---------------------------------------------------------------------------
void __fastcall Tieee::Button13Click(TObject *Sender)  // Bandpass
{
    char st[40];
    sprintf(st,"%s",Edit2->Text.c_str());
    Nr_channels = atof(st);
    if ((Nr_channels <1) || (Nr_channels > 13200))
    {
       ShowMessage("Number of channels out of range");
       return;
    }

    sprintf(st,"%s",Edit16->Text.c_str());
    integration = atoi(st);
    if ((integration <1) || (integration > 1000))
    {
       ShowMessage("Number of integrations out of range");
       return;
    }

    sprintf(st,"%s",Edit4->Text.c_str());
    fmax = atof(st);

    sprintf(st,"%s",Edit5->Text.c_str());
    fmin = atof(st);
    f = fmin;
    df = (fmax - fmin)/(float)Nr_channels;

    if ((fmin<45.0) || (fmin>870.0) || (fmax<45.0) || (fmax>870.0) || (fmin>fmax))
    {
       ShowMessage("Frequencies out of range");
       return;
    }

    pwm = atoi(Edit3->Text.c_str());
    if ((pwm <1) || (pwm > 255))
    {
       ShowMessage("PWM value out of range");
       return;
    }
    sprintf(st,"O%03i\r",pwm); // set gain
    RX_SerialThread->Send(st);

    Form1->Close();
    Form2->Close();
    Form3->Close();
    Reset();
    Erase();
    Timer6->Enabled = true;
}
//---------------------------------------------------------------------------
void __fastcall Tieee::Timer6Timer(TObject *Sender) // Bandpass auto state machine
{
   static int state = 0;
   Button1->Enabled = false; // cold
   Button2->Enabled = false; // hot
   Button4->Enabled = false; // plot NF
   Button5->Enabled = false; // disconnect
   Button6->Enabled = false; // warm
   Button7->Enabled = false; // plot mV/dB
   Button8->Enabled = false; // plot raw
   Button9->Enabled = false; // auto nf
   Button12->Enabled = false; // update config

   if (stopped2)
   {
      state = 0;
      stopped2 = false;
   }

   switch (state)
   {
      case 0:
      {
         Button1Click(0); // cold
         state++;
         break;
      }
      case 1:
      {
         if (processstate==1)
            state++;
         break;
      }
      case 2:
      {
         //Button6Click(0); // warm
         //state++;
         state = 4; // ignore warm
         break;
      }
      case 3:
      {
         if (processstate==2)
           state++;
         break;
      }
      case 4:
      {
         Button2Click(0); // hot
         state++;
         break;
      }
      case 5:
      {
         if (processstate==3)
         {
            Timer2->Enabled = false;
            Button14Click(0);  // plott BPF
            Button1->Enabled = true; // cold
            Button2->Enabled = true; // hot
            Button4->Enabled = true; // plot NF
            Button5->Enabled = true; // disconnect
            Button6->Enabled = true; // warm
            Button7->Enabled = true; // plot mV/dB
            Button8->Enabled = true; // plot raw
            Button9->Enabled = true; // auto nf
            Button12->Enabled = true; // update config
            state = 0;
         }
         break;
      }
   }
}
//---------------------------------------------------------------------------
void __fastcall Tieee::Button14Click(TObject *Sender)  // Plot BPF
{
    char Title[bufferwidth], st[bufferwidth], Parameter[bufferwidth],
         subtitle[bufferwidth], file[bufferwidth], extension[5];

    Word Year, Month, Day, Hour, Min, Sec, MSec;

    sprintf(st,"%s",Edit17->Text.c_str());
    gradient = atof(st);
    if (gradient < 1.0) gradient = 1.0;

    MyPlott->SetNumber(anzMess);
    MyPlott->SetXvector(xarr_all);

    for (int i=0; i<anzMess; i++)  // calculate dB
    {
       bpf[i] = abs(yarr_hot[i] - yarr_cold[i])/gradient; // Y-dB
    }
    float maxi = -1000.0;
    float mini = 1000.0;
    for (int i=0; i<anzMess; i++) // search extrema
    {
       if (bpf[i] > maxi) maxi = bpf[i];
       if (bpf[i] < mini) mini = bpf[i];
    }
    for (int i=0; i<anzMess; i++) // normalize
    {
       bpf[i] = bpf[i] - maxi;
    }
    MyPlott->SetYvector1(bpf);

    sprintf(Title,"Overall band pass e-Callisto #%s",Edit11->Text.c_str());
    MyPlott->SetTitle(Title);

    sprintf(Parameter,"pwm=%3i digit",pwm);
    MyPlott->SetSubTitle(Parameter);

    sprintf(subtitle,"...");
    MyPlott->SetSubSubTitle(subtitle);

    int xticks = atoi(Edit15->Text.c_str());
    MyPlott->SetXminmax(fmin, fmax, (fmax-fmin)/xticks);

    MyPlott->SetXLabel("Frequency [MHz]");
    MyPlott->SetYLabel("Attenuation [dB]");

    MyPlott->SetThickness_Curve(atoi(Edit7->Text.c_str()));
    MyPlott->SetMarker(atoi(Edit8->Text.c_str()));
    MyPlott->SetThickness_Marker(atoi(Edit10->Text.c_str()));

    sprintf(st,"%s",Edit9->Text.c_str());
    MyPlott->SetColor1(st);

    MyPlott->SetBGColor(atoi(Edit18->Text.c_str()));

    float ymin = atof(Edit14->Text.c_str());
    float ymax = atof(Edit13->Text.c_str());
    int yticks = atoi(Edit6->Text.c_str());
    MyPlott->SetYminmax(ymin, ymax , (ymax-ymin)/yticks); // lineare skala

    if (RadioButton1->Checked) {MyPlott->SetDevice("XWIN"); sprintf(extension,"%s","nil");}
    if (RadioButton2->Checked) {MyPlott->SetDevice("PDF");  sprintf(extension,"%s","PDF");}
    if (RadioButton3->Checked) {MyPlott->SetDevice("EPS");  sprintf(extension,"%s","EPS");}
    if (RadioButton4->Checked) {MyPlott->SetDevice("TIFF"); sprintf(extension,"%s","TIF");}
    if (RadioButton5->Checked) {MyPlott->SetDevice("GIF");  sprintf(extension,"%s","GIF");}
    if (RadioButton6->Checked) {MyPlott->SetDevice("BMP");  sprintf(extension,"%s","BMP");}
    if (RadioButton7->Checked) {MyPlott->SetDevice("PNG");  sprintf(extension,"%s","PNG");}

    TDateTime dtPresent = Now();
    DecodeDate(dtPresent, Year, Month, Day);

    DecodeTime(dtPresent, Hour, Min, Sec, MSec);

    sprintf(file,"BPF_%s_PWM%3i_%04u-%02u-%02u_%02u%02u%02u.%s",
            Edit11->Text.c_str(),pwm,Year, Month, Day,Hour, Min, Sec,extension);
    MyPlott->SetFilename(file);

    MyPlott->LinPlott(true); // single

    Timer6->Enabled=false;
}
//---------------------------------------------------------------------------

